import numpy as np
import shutil
import os
import csv
from datetime import datetime, timedelta
import pandas as pd
import os
import sys
from model_conf import *
import sqlite3
import subprocess
import json
from model_conf import input_data,geners
import re

"""
Se encarga de la extraccion de informacion de un archivo de salida TOUGH2 v6.x
"""

def write_PT_from_t2output(db_path=input_data['db_path'],wells=[i for x in ['WELLS','MAKE_UP_WELLS','NOT_PRODUCING_WELL'] for i in input_data[x]]):
	"""Escribe la ultima linea de salida del archivo t2.out relativa a cada bloque de pozo

	Parameters
	----------
	db_path : str
	  Direccion de base de datos sqlite, tomado de model_conf

	Returns
	-------
	file
	  {well_name}_PT.txt: archivo de texto que contiene la informacion relacionado con bloques de todas la capas relacionados a un pozo

	Attention
	---------
	Genera los bloques a extraer de la base de datos sqlite

	Note
	----
	Se auxilia del archivo shell shell/write_PT.sh
	Examples
	--------
	>>> write_PT_from_t2output(db_path)
	"""

	t2_output_file="../model/t2/t2.out"
	if os.path.isfile(t2_output_file):
		pass
	else:
		return "Theres is not t2.out file on t2/t2.out"

	output_headers=[]
	with open(t2_output_file,'r') as t2_file:
		t2_output_array = t2_file.readlines()
		for line_i, line in enumerate(t2_output_array):
			if "OUTPUT DATA AFTER" in line.rstrip():
				output_headers.append(line_i)

	conn=sqlite3.connect(db_path)
	c=conn.cursor()

	#Listing all the wells and assigning them to a well

	blocks_wells={}
	wells_data={}

	data_layer=pd.read_sql_query("SELECT correlative FROM layers ORDER BY  middle DESC;",conn)

	for name in sorted(wells):
		wells_data[name]=""
		data_block=pd.read_sql_query("SELECT blockcorr FROM t2wellblock  WHERE well='%s' ORDER BY blockcorr;"%name,conn)
		if len(data_block)>0:
			for n in data_layer['correlative'].values:
				blocks_wells[n+data_block['blockcorr'].values[0]]=name
	
	for line in t2_output_array[output_headers[-1]:-1]:
		for block in blocks_wells:
			if  block in line.split() and len(line.split())==11:
				wells_data[blocks_wells[block]]+="%s\n"%(','.join(line.split()))

	for well in wells_data:
		file_out=open("../output/PT/txt/%s_PT.dat"%(well), "w")
		file_out.write("ELEM,INDEX,P,T,SG,SW,X(WAT1),X(WAT2),PCAP,DG,DW\n")
		file_out.write(wells_data[well])
		file_out.close()

	conn.close()
	
def from_sav_to_json(sav_version='sav1'):
	"""Escribre un json con las propiedades de salida de los archivo .sav o .sav1 (P,T) para cada bloque, identificandolos de forma espacial
	
	Parameters
	----------
	src_file : str
	  Nombre de archivo sav, puede ser t2.sav o t2.sav1

	Returns
	-------
	file
	  PT_json_from_sav.txt : en direccion  "../output/PT/json/PT_json_from_sav.txt"

	Attention
	---------
	Utiliza el archivo "../mesh/to_steinar/in" como criterio para atribuir coordenadas a cada bloque

	Examples
	--------
	>>> from_sav_to_json("t2.sav1")
	"""

	output_sav_file="../model/t2/t2.%s"%sav_version

	with open('../mesh/ELEME.json') as file:
	  	blocks_position=json.load(file)

	eleme_dict={}
	for block in blocks_position:
		eleme_dict[block]=[blocks_position[block]['X'],blocks_position[block]['Y'],blocks_position[block]['Z']]

	#Creates a string from .sav file
	savfile=open(output_sav_file,"r")
	savstring=[]
	for linesav in savfile:
		savstring.append(linesav.rstrip())
	savfile.close()

	if os.path.isfile(output_sav_file):
		t2_sav_file=open(output_sav_file, "r")
		contidion_found=False
		for t2_sav_line in t2_sav_file:
			if  t2_sav_line[0:5] in  eleme_dict.keys():
				contidion_found=True
				block=t2_sav_line[0:5]
				continue
			if contidion_found:
				eleme_dict[block].extend([float(i) for i in t2_sav_line.split()])
				contidion_found=False
		t2_sav_file.close()

	eleme_pd=pd.DataFrame.from_dict(eleme_dict,orient='index', columns=['X', 'Y','Z','P','T'])
	eleme_pd.to_json("../output/PT/json/PT_json_from_sav.txt",orient="index",indent=2)

def PTjson_to_sqlite(source='t2',db_path=input_data['db_path']):
	"""Escribe la salida de extract_json_from_t2out o from_sav_to_json a base de datos sqlite
	
	Parameters
	----------
	source : str
	  Puede ser t2 o sav

	Note
	----
	Escribe en la tabla t2PTout

	Examples
	--------
	>>> PTjson_to_sqlite("t2")
	"""

	conn=sqlite3.connect(db_path)
	c=conn.cursor()
	if source=="t2":
		if os.path.isfile("../output/PT/json/PT_json.txt"):
			with open("../output/PT/json/PT_json.txt") as f:
				data=json.load(f)
			for element in sorted(data):
				try:
					q="INSERT INTO t2PTout(blockcorr,x,y,z,'index',P,T,SG,SW,X1,X2,PCAP,DG,DW) \
					VALUES ('%s',%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s)"%(element,	data[element]['X'],data[element]['Y'],data[element]['Z'],data[element]['INDEX'],\
																			data[element]['P'],data[element]['T'],data[element]['SG'],\
																			data[element]['SW'],data[element]['X(WAT1)'],data[element]['X(WAT2)'],\
																			data[element]['PCAP'],data[element]['DG'],data[element]['DW'])

					c.execute(q)
				except sqlite3.IntegrityError:
					q="UPDATE t2PTout SET \
					x=%s, \
					y=%s, \
					z=%s, \
					'index'=%s, \
					P=%s, \
					T=%s, \
					SG=%s, \
					SW=%s, \
					X1=%s, \
					X2=%s, \
					PCAP=%s, \
					DG=%s, \
					DW=%s \
					WHERE blockcorr='%s'"%(	data[element]['X'],data[element]['Y'],data[element]['Z'],data[element]['INDEX'],\
											data[element]['P'],data[element]['T'],data[element]['SG'],\
											data[element]['SW'],data[element]['X(WAT1)'],data[element]['X(WAT2)'],\
											data[element]['PCAP'],data[element]['DG'],data[element]['DW'],element)
					c.execute(q)
				conn.commit()
	elif source=="sav":
		if os.path.isfile("../output/PT/json/PT_json_from_sav.txt"):
			with open("../output/PT/json/PT_json_from_sav.txt") as f:
				data=json.load(f)
			for element in sorted(data):
				try:
					q="INSERT INTO t2PTout(blockcorr,x,y,z,P,T) \
					VALUES ('%s',%s,%s,%s,%s,%s)"%(element,\
												   data[element]['X'],data[element]['Y'],data[element]['Z'],\
												   data[element]['P'],data[element]['T'])
					c.execute(q)
				except sqlite3.IntegrityError:
					q="UPDATE t2PTout SET \
					x=%s, \
					y=%s, \
					z=%s, \
					P=%s, \
					T=%s \
					WHERE blockcorr='%s'"%(data[element]['ELEM'],\
										   data[element]['X'],data[element]['Y'],data[element]['Z'],\
										   data[element]['P'],data[element]['T'],data[element]['ELEM'])
					c.execute(q)
				conn.commit()

	conn.close()

def write_PT_of_wells_from_t2output_in_time(db_path=input_data['db_path'],wells=[i for x in ['WELLS','MAKE_UP_WELLS','NOT_PRODUCING_WELL'] for i in input_data[x]]):
	"""Extrae la evolucion de los bloques relacionados de todos los pozos en la direccion "../output/PT/evol"
	
	Parameters
	----------
	db_path : str
	  Direccion de base de datos sqlite, tomado de model_conf

	Note
	----
	Se auxilia del archivo shell blocks_evol_times.sh

	Examples
	--------
	>>> write_PT_of_wells_from_t2output_in_time(db_path)
	"""

	conn=sqlite3.connect(db_path)
	c=conn.cursor()

	blocks_wells={}
	blocks_data={}

	data_layer=pd.read_sql_query("SELECT correlative FROM layers ORDER BY  middle DESC;",conn)

	for name in sorted(wells):
		data_block=pd.read_sql_query("SELECT blockcorr FROM t2wellblock  WHERE well='%s' ORDER BY blockcorr;"%name,conn)
		if len(data_block)>0:
			for n in data_layer['correlative'].values:
				blocks_wells[n+data_block['blockcorr'].values[0]]=name
				blocks_data[n+data_block['blockcorr'].values[0]]=""

	output_t2_file="../model/t2/t2.out"
	if os.path.isfile(output_t2_file):
		t2_file=open(output_t2_file, "r")
		for t2_line in t2_file:
			if "OUTPUT DATA AFTER" in t2_line:
				time=t2_line.rstrip().split(" ")[-2]
			if len(t2_line.split())==11 and t2_line.split()[0] in blocks_data.keys():
				t2_array=t2_line.rstrip().split(" ")
				data_list= list(filter(None, t2_array))
				data_list.append(time)
				blocks_data[t2_line.split()[0]]+=','.join(data_list)+'\n'
		t2_file.close()
	else:
		sys.exit("The file %s or directory do not exist"%output_t2_file)

	for block in blocks_data:
		evol_file_out=open("../output/PT/evol/%s_PT_%s_evol.dat"%(blocks_wells[block],block[0]), "w")
		evol_file_out.write("ELEM,INDEX,P,T,SG,SW,X(WAT1),X(WAT2),PCAP,DG,DW,TIME\n")
		evol_file_out.write(blocks_data[block])
		evol_file_out.close()	
	conn.close()

def gen_evol():
	"""Extrae la evolucion (Flujo y entalpia) de las fuentes o sumideros "GEN" en la direccion "../output/mh/txt"

	Note
	----
	Se auxilia del archivo shell gen_evol.sh

	Examples
	--------
	>>> gen_evol()
	"""
	"""Extrae la evolucion (Flujo y entalpia) de las fuentes o sumideros "SRC" (Estas se relacionan con cada pozo)en la direccion "../output/mh/txt"
	
	Note
	----
	Se auxilia del archivo shell src_evol.sh

	Examples
	--------
	>>> src_evol()
	"""

	conn=sqlite3.connect(input_data['db_path'])
	c=conn.cursor()
	data_source=pd.read_sql_query("SELECT well,blockcorr,source_nickname FROM t2wellsource WHERE  source_nickname LIKE'GEN*' ORDER BY source_nickname;",conn)

	final_t2=""
	output_fi_file="../model/t2/t2.out"

	dictionary_files={}

	for n in range(len(data_source)):
		well=data_source['well'][n]
		blockcorr=data_source['blockcorr'][n]
		source=data_source['source_nickname'][n]
		dictionary_files[well]={'filename':"../output/mh/txt/%s_%s_%s_evol_mh.dat"%(well,blockcorr,source),'file_container':"",'blockcorr':blockcorr,'source':source}
		dictionary_files[well]['file_container']+="ELEMENT,SOURCEINDEX,GENERATION RATE,ENTHALPY,X1,X2,FF(GAS),FF(AQ.),P(WB),TIME\n"

	output_t2_file="../model/t2/t2.out"
	if os.path.isfile(output_t2_file):
		t2_file=open(output_t2_file, "r")
		for t2_line in t2_file:
			if "OUTPUT DATA AFTER" in t2_line:
				time=t2_line.rstrip().split(" ")[-2]
			for well in dictionary_files:
				if dictionary_files[well]['blockcorr'] in t2_line and dictionary_files[well]['source'] in t2_line:
					t2_array=t2_line.rstrip().split(" ")
					str_list = list(filter(None, t2_array))
					str_list.append(time)
					dictionary_files[well]['file_container']+=','.join(str_list)+'\n'
		t2_file.close()
	else:
		sys.exit("The file %s or directory do not exist"%output_t2_file)

	for well in dictionary_files:
		t2_file_out=open(dictionary_files[well]['filename'], "w")
		t2_file_out.write(dictionary_files[well]['file_container'])
		t2_file_out.close()	

	conn.close()

def src_evol():
	"""Extrae la evolucion (Flujo y entalpia) de las fuentes o sumideros "SRC" (Estas se relacionan con cada pozo)en la direccion "../output/mh/txt"
	
	Note
	----
	Se auxilia del archivo shell src_evol.sh

	Examples
	--------
	>>> src_evol()
	"""

	conn=sqlite3.connect(input_data['db_path'])
	c=conn.cursor()
	data_source=pd.read_sql_query("SELECT well,blockcorr,source_nickname FROM t2wellsource WHERE  source_nickname NOT LIKE'GEN*' ORDER BY source_nickname;",conn)

	final_t2=""
	output_fi_file="../model/t2/t2.out"

	dictionary_files={}

	for n in range(len(data_source)):
		well=data_source['well'][n]
		blockcorr=data_source['blockcorr'][n]
		source=data_source['source_nickname'][n]
		dictionary_files[well]={'filename':"../output/mh/txt/%s_%s_%s_evol_mh.dat"%(well,blockcorr,source),'file_container':"",'blockcorr':blockcorr,'source':source}
		dictionary_files[well]['file_container']+="ELEMENT,SOURCEINDEX,GENERATION RATE,ENTHALPY,X1,X2,FF(GAS),FF(AQ.),P(WB),TIME\n"

	output_t2_file="../model/t2/t2.out"
	if os.path.isfile(output_t2_file):
		t2_file=open(output_t2_file, "r")
		for t2_line in t2_file:
			if "OUTPUT DATA AFTER" in t2_line:
				time=t2_line.rstrip().split(" ")[-2]
			for well in dictionary_files:
				if dictionary_files[well]['blockcorr'] in t2_line and dictionary_files[well]['source'] in t2_line:
					t2_array=t2_line.rstrip().split(" ")
					str_list = list(filter(None, t2_array))
					str_list.append(time)
					dictionary_files[well]['file_container']+=','.join(str_list)+'\n'
		t2_file.close()
	else:
		sys.exit("The file %s or directory do not exist"%output_t2_file)

	for well in dictionary_files:
		t2_file_out=open(dictionary_files[well]['filename'], "w")
		t2_file_out.write(dictionary_files[well]['file_container'])
		t2_file_out.close()	

	conn.close()

def write_PT_from_t2output_from_prod(db_path=input_data['db_path'],wells=[i for x in ['WELLS','MAKE_UP_WELLS','NOT_PRODUCING_WELL'] for i in input_data[x]],sav_version='sav1'):
	"""Escribe la presion y temperatura para un pozo a partir de la salida del estado de produccion, utiliza el archivo .sav1, generado a partir del uso de STEADY-STATE y  RESTART TIME en el archivo itough2

	Parameters
	----------
	db_path : str
	  Direccion de base de datos sqlite, tomado de model_conf

	Returns
	-------
	file
	  {well_name}_PT.txt: archivo de texto que contiene la informacion relacionado con bloques de todas la capas relacionados a un pozo

	Attention
	---------
	Genera los bloques a extraer de la base de datos sqlite

	Note
	----
	Se auxilia del archivo shell shell/write_PT_from_prod.sh
	Examples
	--------
	>>> write_PT_from_t2output_from_prod(db_path)
	"""


	conn=sqlite3.connect(db_path)
	c=conn.cursor()

	#Listing all the wells and assigning them to a well

	blocks_wells={}
	wells_data={}
	blocks_dictionary={}

	data_layer=pd.read_sql_query("SELECT correlative FROM layers ORDER BY  middle DESC;",conn)

	for name in sorted(wells):
		wells_data[name]=""
		data_block=pd.read_sql_query("SELECT blockcorr FROM t2wellblock  WHERE well='%s' ORDER BY blockcorr;"%name,conn)
		if len(data_block)>0:
			for n in data_layer['correlative'].values:
				blocks_wells[n+data_block['blockcorr'].values[0]]=name

	
	output_sav_file="../model/t2/t2.%s"%sav_version
	if os.path.isfile(output_sav_file):
		t2_sav_file=open(output_sav_file, "r")
		contidion_found=False
		for t2_sav_line in t2_sav_file:
			if  t2_sav_line[0:5] in  blocks_wells.keys():
				contidion_found=True
				well=blocks_wells[t2_sav_line[0:5]]
				block=t2_sav_line[0:5]
				continue
			if contidion_found:
				wells_data[well]+="%s%s"%(block,','.join(t2_sav_line.split(" ")))
				contidion_found=False
		t2_sav_file.close()

	for well in wells_data:
		file_out=open("../output/PT/txt/%s_PT.dat"%(well), "w")
		file_out.write("ELEM,P,T\n")
		file_out.write(wells_data[well])
		file_out.close()

	conn.close()

def extract_from_t2out(json_output=False):
	"""Escribre un csv con todas las propiedades de salida (ELEM,P,T,SG,SW,X(WAT1),X(WAT2),PCAP,DG,DW) del archivo t2.out para cada bloque, junto con su identificacion espacial

	Returns
	-------
	file
	  PT.csv : en direccion  '../output/PT/csv/PT_json.txt'

	Attention
	---------
	Utiliza el archivo "../mesh/to_steinar/in" como criterio para atribuir coordenadas a cada bloque

	Examples
	--------
	>>> extract_json_from_t2out(db_path)
	"""
	
	eleme_dict={}

	ELEME_file='../mesh/ELEME.json'
	if os.path.isfile(ELEME_file):
		with open(ELEME_file) as file:
		  	blocks_position=json.load(file)

		for block in blocks_position:
			eleme_dict[block]=[blocks_position[block]['X'],blocks_position[block]['Y'],blocks_position[block]['Z']]
	else:
		return "The file %s does not exist"%ELEME_file

	last=""
	if os.path.isfile("../model/t2/t2.out"):
		t2file=open("../model/t2/t2.out","r")
	else:
		return "Theres is not t2.out file on t2/t2.out"


	cnt=0
	t2string=[]
	#It finds the last section where OUTPUT DATA AFTER was printed and uses it to know where to start to extract data. But also converts every line of the file into an element in an array
	for linet2 in t2file:
		cnt+=1
		t2string.append(linet2.rstrip())
		if "OUTPUT DATA AFTER" in linet2.rstrip():
			last=linet2.rstrip().split(",")
			line=cnt
	t2file.close()

	high_iteration=[int(s) for s in last[0].split() if s.isdigit()]

	for elementx in eleme_dict:
		cnt2=0
		for lineout in t2string[line+cnt2:-1]:
			if " @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@"==lineout:
				cnt2+=1
			elif cnt2>2:
				break
			elif elementx in lineout:
				lineselect=lineout.split()
				eleme_dict[elementx].extend(lineselect)
				break

	csv_columns=['X','Y','Z','ELEM','INDEX','P','T','SG','SW','X(WAT1)','X(WAT2)','PCAP','DG','DW']

	if json_output:
		eleme_pd=pd.DataFrame.from_dict(eleme_dict,orient='index',columns=csv_columns)
		dtype= {'X':'float',
				 'Y':'float',
				 'Z':'float',
				 'ELEM':'str',
				 'INDEX':'float',
				 'P':'float',
				 'T':'float',
				 'SG':'float',
				 'SW':'float',
				 'X(WAT1)':'float',
				 'X(WAT2)':'float',
				 'PCAP':'float',
				 'DG':'float',
				 'DW':'float'}
		eleme_pd= eleme_pd.astype(dtype)
		eleme_pd.to_json("../output/PT/json/PT_json.txt",orient="index",indent=2)

	with open("../output/PT/csv/PT.csv",'w') as file:
		file.write(','.join(csv_columns))
		file.write('\n')
		for key in eleme_dict.keys():
			string=""
			for d in eleme_dict[key]:
				string+="%s,"%(d)
			file.write(string[0:-2])
			file.write('\n')
	file.close()

#extract_json_from_t2out()

#from_sav_to_json("t2.sav1")

#write_PT_of_wells_from_t2output_in_time(db_path)
#PTjson_to_sqlite(source="t2")#source="sav",t2

#gen_evol()

#src_evol()


"""
CHINAMECA
"""

#write_PT_from_t2output(db_path)

#extract_json_from_t2out()

#src_evol()
#write_PT_of_wells_from_t2output_in_time(db_path)

#Sostenibilidad Ahuachapan

#**---Comparacion en estado natural

#write_PT_from_t2output(db_path)

#**--

#**--Comparacion completa de avance, en estado de produccion

#gen_evol()

#src_evol()

#write_PT_of_wells_from_t2output_in_time(db_path)

#write_PT_from_t2output_from_prod(db_path)

#from_sav_to_json("t2.sav1")

#**--

#extract_csv_from_t2out()

#src_evol()

#gen_evol()

#write_PT_from_t2output_from_prod(json_output=True)

#extract_csv_from_t2out()

#write_PT_from_t2output()

#from_sav_to_json()

#write_PT_of_wells_from_t2output_in_time()

#extract_from_t2out(json_output=True)

#PTjson_to_sqlite(source='sav')


import sqlite3
import os
import pandas as pd
import json
from model_conf import *
import geometry as geomtr
from model_conf import input_data

source_txt='../input/'

def checktable(table_name,c):
	"""Verifica la existencia de una tabla en la base de datos sqlite model.db 

	Parameters
	----------
	table_name : str
	  Nombre de tabla
	c : cursor
	  Conexion a la base de datos

	Returns
	-------
	int
		check: 1 si la tabla existe y 0 si la tabla no existe

	Examples
	--------
	>>>  checktable(table_name,c)
	"""

	query="SELECT COUNT(name) from sqlite_master WHERE type='table' AND name='%s'"%(table_name)
	c.execute(query)
	if c.fetchone()[0]==1:
		check=1
	else:
		check=0
	return check

def db_creation(db_path=input_data['db_path']):
	"""Genera base de datos sqlite model.db 

	Parameters
	----------
	db_path : str
	  Direccion de base de datos sqlite, tomado de model_conf

	Returns
	-------
	database
		model.db: base de datos '../input/model.db'

	Note
	----
	Se generan las tablas: wells, survey, PT, mh, drawdown, cooling, wellfeedzone, t2wellblock, t2wellsource, layers y t2PTout


	Examples
	--------
	>>> db_creation(db_path)
	"""

	if not os.path.isfile(db_path):
		conn=sqlite3.connect(db_path)
		c = conn.cursor()

		# Create table - wells
		if checktable('wells',c)==0:
			c.execute('''CREATE TABLE wells
		             ([well] TEXT PRIMARY KEY,
		              [type] TEXT,
		              [east] REAL,
		              [north] REAL,
		              [elevation] REAL,
		              [drilldate] datetime)''')

		#Create table - survey
		if checktable('survey',c)==0:
			c.execute('''CREATE TABLE  survey
						 ([well] TEXT,
						 [MeasuredDepth] REAL,
						 [Delta_east] REAL,
						 [Delta_north] REAL)''')

		#Create table - PT
		if checktable('PT',c)==0:
			c.execute('''CREATE TABLE  PT
						 ([well] TEXT,
						 [MeasuredDepth] REAL,
						 [Pressure] REAL,
						 [Temperature] REAL)''')

		#Create table - mh
		if checktable('mh',c)==0:
			c.execute('''CREATE TABLE  mh
						 ([well] TEXT,
						 [type] TEXT,
						 [date_time] datetime,
						 [steam_flow] REAL,
						 [liquid_flow] REAL,
						 [flowing_enthalpy] REAL,
						 [well_head_pressure] REAL)''')

		#Create table - drawdown
		if checktable('drawdown',c)==0:
			c.execute('''CREATE TABLE  drawdown
						 ([well] TEXT,
						 [date_time] datetime,
						 [TVD] REAL,
						 [pressure] REAL)''')

		#Create table - cooling
		if checktable('cooling',c)==0:
			c.execute('''CREATE TABLE  cooling
						 ([well] TEXT,
						 [date_time] datetime,
						 [TVD] REAL,
						 [temp] REAL)''')

		#Create table - wellfeedzone
		if checktable('wellfeedzone',c)==0:
			c.execute('''CREATE TABLE  wellfeedzone
						 ([well] TEXT,
						 [MeasuredDepth] REAL,
						 [porcentage] REAL)''')

		#Create table - TOUGH2 well block(correlative)
		if checktable('t2wellblock',c)==0:
			c.execute('''CREATE TABLE  t2wellblock
						 ([well] TEXT PRIMARY KEY,
						 [blockcorr] TEXT)''')

		#Create table - TOUGH2 well source
		if checktable('t2wellsource',c)==0:
			c.execute('''CREATE TABLE  t2wellsource
						 ([well] TEXT,
						 [blockcorr] TEXT ,
						 [source_nickname] TEXT PRIMARY KEY)''')

		#Create table - layers levels
		if checktable('layers',c)==0:
			c.execute('''CREATE TABLE  layers
						 ([correlative] TEXT PRIMARY KEY,
						 [top] REAL,
						 [middle] REAL,
						 [bottom] REAL)''')

		#Create table - PT out
		if checktable('t2PTout',c)==0:
			c.execute('''CREATE TABLE  t2PTout
						 ([blockcorr] TEXT PRIMARY KEY,
						 [x] REAL,
						 [y] REAL,
						 [z] REAL,
						 [index] REAL,
						 [P] REAL,
						 [T] REAL,
						 [SG] REAL,
						 [SW] REAL,
						 [X1] REAL,
						 [X2] REAL,
						 [PCAP] REAL,
						 [DG] REAL,
						 [DW] REAL)''')

		conn.commit()
		conn.close()

def insert_wells_sqlite(db_path=input_data['db_path'],source_txt=source_txt):
	"""Toma la informacion contenida en el archivo ../input/ubication.csv

	Parameters
	----------
	db_path : str
	  Direccion de base de datos sqlite, tomado de model_conf
	source_txt : str
	  Direccion de folder con datos fuentes

	Note
	----
	Escribe los datos en la tabla wells, no permite actualizaciones, por lo que de modificar el archivo ubication.csv, es preferible eliminar registros en base y volver a ejecutar esta funcion

	Examples
	--------
	>>>  insert_wells_sqlite(db_path,'../input/')
	"""
	conn=sqlite3.connect(db_path)
	c = conn.cursor()

	wells=pd.read_csv(source_txt+'ubication.csv')

	wells['drilldate'] = pd.to_datetime(wells['drilldate'],format="%Y%m%d")

	for  index,row in wells.iterrows():
		try:
			q="INSERT INTO wells(well,type,east,north,elevation,drilldate) VALUES ('%s','%s',%s,%s,%s,'%s')"%\
			(row['well'],row['type'],row['east'],row['north'],row['masl'],row['drilldate'])
			c.execute(q)
			conn.commit()
		except sqlite3.IntegrityError:
			print("The well %s is already on the database")
	conn.close()

def insert_feedzone_to_sqlite(db_path=input_data['db_path'],source_txt=source_txt):
	"""Toma la informacion contenida en el archivo ../input/well_feedzone.csv

	Parameters
	----------
	db_path : str
	  Direccion de base de datos sqlite, tomado de model_conf
	source_txt : str
	  Direccion de folder con datos fuentes

	Note
	----
	Escribe los datos en la tabla wellfeedzone, no permite actualizaciones, por lo que de modificar el archivo well_feedzone.csv, es preferible eliminar registros en base y volver a ejecutar esta funcion

	Examples
	--------
	>>>  insert_feedzone_to_sqlite(db_path,'../input/')
	"""

	conn=sqlite3.connect(db_path)
	c=conn.cursor()

	feedzones=pd.read_csv(source_txt+'well_feedzone.csv',delimiter=',')

	for index,row in feedzones.iterrows():
		q="INSERT INTO wellfeedzone(well,MeasuredDepth,porcentage) VALUES ('%s',%s,%s)"%\
		(row['well'],row['MeasuredDepth'],row['porcentage'])
		print(q)
		c.execute(q)
		conn.commit()
	conn.close()

def insert_survey_to_sqlite(db_path=input_data['db_path'],source_txt=source_txt):
	"""Toma la informacion contenida en los archivos de la carpeta  ../input/survey

	Parameters
	----------
	db_path : str
	  Direccion de base de datos sqlite, tomado de model_conf
	source_txt : str
	  Direccion de folder con datos fuentes

	Note
	----
	Escribe los datos en la tabla survey, no permite actualizaciones, por lo que de modificar los archivos contenidos en survey, es preferible eliminar registros en base y volver a ejecutar esta funcion

	Examples
	--------
	>>>  insert_survey_to_sqlite(db_path,'../input/')
	"""
	
	conn=sqlite3.connect(db_path)
	c=conn.cursor()
	
	for f in os.listdir(source_txt+'survey/'):
		if os.path.isfile(os.path.join(source_txt, 'survey/',f)):
			well_name=f.replace("'","").replace("_MD.dat","")
			well_file=os.path.join(source_txt, 'survey/',f)
			survey=pd.read_csv(well_file)
			for index, row in survey.iterrows():
				q="INSERT INTO survey(well,MeasuredDepth,Delta_north,Delta_east) VALUES ('%s',%s,%s,%s)"%\
				(well_name,row['MeasuredDepth'],row['Delta_north'],row['Delta_east'])
				c.execute(q)
				conn.commit()
	conn.close()

def insert_PT_to_sqlite(db_path=input_data['db_path'],source_txt=source_txt):
	"""Toma la informacion contenida en los archivos de la carpeta  ../input/survey

	Parameters
	----------
	db_path : str
	  Direccion de base de datos sqlite, tomado de model_conf
	source_txt : str
	  Direccion de folder con datos fuentes

	Note
	----
	Escribe los datos en la tabla PT, no permite actualizaciones, por lo que de modificar los archivos contenidos en la carpeta PT, es preferible eliminar registros en base y volver a ejecutar esta funcion

	Examples
	--------
	>>>  insert_PT_to_sqlite(db_path,'../input/')
	"""

	conn=sqlite3.connect(db_path)
	c=conn.cursor()
	for f in os.listdir(source_txt+'PT'):
		if os.path.isfile(source_txt+'PT/'+f):
			if '_MDPT' in f:
				well_name=f.replace("'","").replace("_MDPT.dat","")
				if os.path.isfile(source_txt+'PT/'+f):
					PT=pd.read_csv(source_txt+'PT/'+f)
					for index, row in PT.iterrows():
						q="INSERT INTO PT(well,MeasuredDepth,Pressure,Temperature) VALUES ('%s',%s,%s,%s)"%\
						(well_name,row['MD'],row['P'],row['T'])
						c.execute(q)
						conn.commit()
	conn.close()

def insert_drawdown_to_sqlite(db_path=input_data['db_path'],source_txt=source_txt):
	"""Toma la informacion contenida en los archivos de la carpeta  ../input/drawdown

	Parameters
	----------
	db_path : str
	  Direccion de base de datos sqlite, tomado de model_conf
	source_txt : str
	  Direccion de folder con datos fuentes

	Note
	----
	Escribe los datos en la tabla drawdown, no permite actualizaciones, por lo que de modificar los archivos contenidos en la carpeta drawdown, es preferible eliminar registros en base y volver a ejecutar esta funcion

	Examples
	--------
	>>>  insert_drawdown_to_sqlite(db_path,'../input/')
	"""

	conn=sqlite3.connect(db_path)
	c=conn.cursor()
	for f in os.listdir(source_txt+'drawdown'):
		well_name=f.replace("'","").replace("_DD.dat","")
		if os.path.isfile(source_txt+'drawdown/'+f) and f!='p_res.csv':
			C=pd.read_csv(source_txt+'drawdown/'+f)
			for index, row in C.iterrows():
				q="INSERT INTO drawdown(well,date_time,TVD,pressure) VALUES ('%s','%s',%s,%s)"%\
				(well_name,row['datetime'],row['TVD'],row['pressure'])
				c.execute(q)
				conn.commit()
	conn.close()

def insert_cooling_to_sqlite(db_path=input_data['db_path'],source_txt=source_txt):
	"""Toma la informacion contenida en los archivos de la carpeta  ../input/cooling

	Parameters
	----------
	db_path : str
	  Direccion de base de datos sqlite, tomado de model_conf
	source_txt : str
	  Direccion de folder con datos fuentes

	Note
	----
	Escribe los datos en la tabla cooling, no permite actualizaciones, por lo que de modificar los archivos contenidos en la carpeta cooling, es preferible eliminar registros en base y volver a ejecutar esta funcion

	Examples
	--------
	>>>  insert_cooling_to_sqlite(db_path,'../input/')
	"""

	conn=sqlite3.connect(db_path)
	c=conn.cursor()
	for f in os.listdir(source_txt+'cooling'):
		well_name=f.replace("'","").replace("_C.dat","")
		print(well_name)
		if os.path.isfile(source_txt+'cooling/'+f):
			DD=pd.read_csv(source_txt+'cooling/'+f)
			for index, row in DD.iterrows():
				q="INSERT INTO cooling(well,date_time,TVD,temp) VALUES ('%s','%s',%s,%s)"%\
				(well_name,row['datetime'],row['TVD'],row['temperature'])
				c.execute(q)
				conn.commit()
	conn.close()

def insert_mh_to_sqlite(db_path=input_data['db_path'],source_txt=source_txt):
	"""Toma la informacion contenida en los archivos de la carpeta  ../input/mh

	Parameters
	----------
	db_path : str
	  Direccion de base de datos sqlite, tomado de model_conf
	source_txt : str
	  Direccion de folder con datos fuentes

	Note
	----
	Escribe los datos en la tabla mh, no permite actualizaciones, por lo que de modificar los archivos contenidos en la carpeta mh, es preferible eliminar registros en base y volver a ejecutar esta funcion

	Examples
	--------
	>>>  insert_cooling_to_sqlite(db_path,'../input/')
	"""

	conn=sqlite3.connect(db_path)
	c=conn.cursor()
	for f in os.listdir(source_txt+'mh'):
		well_name=f.replace("'","").replace("_mh.dat","")
		if os.path.isfile(source_txt+'mh/'+f):
			mh=pd.read_csv(source_txt+'mh/'+f)
			for index, row in mh.iterrows():
				q="INSERT INTO mh(well,type,date_time,steam_flow,liquid_flow,flowing_enthalpy,well_head_pressure) VALUES ('%s','%s','%s',%s,%s,%s,%s)"%\
				(well_name,row['type'],row['date-time'],row['steam'],row['liquid'],row['enthalpy'],row['WHPabs'])
				c.execute(q)
				conn.commit()
	conn.close()

def sqlite_to_json(db_path=input_data['db_path'],source_txt=source_txt):
	"""Transforma el contenido de la base de datos sqlite mode.db en formato json

	Parameters
	----------
	db_path : str
	  Direccion de base de datos sqlite, tomado de model_conf
	source_txt : str
	  Direccion donde se almacenara el archivo model.json

	Note
	----
	Convierte a las tablas wells, survey, PT, mh, drawdown, wellfeedzone, t2wellblock y t2wellsource en formato json, utilizando como ID el campo well

	Other Parameters
	----------------
	dictionary
		levels : diccionario interno, indica el ID y el segundo parametro para categorizar una tabla

	Examples
	--------
	>>>  sqlite_to_json(db_path,'../input/')
	"""

	levels={'wells':['well'],
			'survey':['well'],
			'PT':['well','MeasuredDepth'],
			'mh':['well','date_time'],
			'drawdown':['well','date_time'],
			'wellfeedzone':['well'],
			't2wellblock':['well'],
			't2wellsource':['well']}

	conn=sqlite3.connect(db_path)
	c=conn.cursor()

	dict_to_json=dict((n,{}) for n in levels.keys())
	
	for lv in levels:
		cnt=0
		for lv2 in levels[lv]:
			
			if len(levels[lv])==1:

				query="SELECT DISTINCT %s from %s"%(lv2,lv)
				c.execute(query)
				rows=c.fetchall()
				dict_keys=[]
				for row in rows:
					dict_keys.append(str(row[0]))

				dict_of_wells_on_table=dict((n,{}) for n in dict_keys)
				dict_to_json[lv]=dict_of_wells_on_table

				query="PRAGMA table_info(%s)"%lv
				c.execute(query)
				rows=c.fetchall()
				colums=[]
				for row in rows:
					colums.append(str(row[1]))

				partq=[]

				q_string=""
				for nv in colums:
					if nv not in levels[lv] :
						partq.append(nv)
						q_string+=" %s,"%(nv)

				for nv in dict_keys:
					query="SELECT %s from %s WHERE %s='%s'"%(q_string[0:-1],lv,lv2,nv)

					conn=sqlite3.connect(db_path)
					conn.row_factory = sqlite3.Row
					c=conn.cursor()
					rows=c.execute(query).fetchall()

					dict_to_json[lv][nv]=[dict(ix) for ix in rows]

			elif len(levels[lv])>1:
				if cnt==0:
					query="SELECT DISTINCT %s from %s"%(lv2,lv)
					prev_key=lv2

					c.execute(query)
					rows=c.fetchall()
					dict_keys=[]
					for row in rows:
						dict_keys.append(str(row[0]))

					dict_of_wells_on_table=dict((n,{}) for n in dict_keys)
					dict_to_json[lv]=dict_of_wells_on_table
					cnt+=1

				else:

					#Extract the colums which are not on the results
					query="PRAGMA table_info(%s)"%lv
					c.execute(query)
					rows=c.fetchall()
					colums=[]
					for row in rows:
						colums.append(str(row[1]))

					partq=[]

					q_string=""
					for nv in colums:
						if nv not in levels[lv] :
							partq.append(nv)
							q_string+=" %s,"%(nv)

					for prev_result in dict_keys:

						query="SELECT DISTINCT %s from %s WHERE %s='%s'"%(lv2,lv,prev_key,prev_result)

						c.execute(query)
						rows=c.fetchall()
						dict_keys=[]
						for row in rows:
							dict_keys.append(str(row[0]))

						dict_of_wells_on_table=dict((n,{}) for n in dict_keys)
						dict_to_json[lv][prev_result]=dict_of_wells_on_table

						for nv in dict_keys:

							query="SELECT %s from %s WHERE %s='%s' AND %s='%s' "%(q_string[0:-1],lv,lv2,nv,prev_key,prev_result)

							conn=sqlite3.connect(db_path)
							conn.row_factory = sqlite3.Row
							c=conn.cursor()
							rows=c.execute(query).fetchall()

							dict_to_json[lv][prev_result][nv]=[dict(ix) for ix in rows]
	
	with open(source_txt+'model.json', 'w') as jlines:
		json.dump(dict_to_json, jlines,indent=4,sort_keys=True)
	

	conn.close()

def insert_wellblock_to_sqlite(db_path=input_data['db_path'],source_txt=source_txt,input_dictionary=input_data):
	"""Escribe en la tabla t2wellblock, toma los pozos definidos en el arreglo wells y del json 'wells_correlative' creado 
	al ejecutar la mesh se escribe el correlativo de cada pozo

	Parameters
	----------
	db_path : str
	  Direccion de base de datos sqlite, tomado de model_conf
	source_txt : str
	  Direccion donde se almacenara el archivo model.json
	wells : array
	  Arreglo de pozos a ser introducidos, tomado de model_conf

	Note
	----
	Si el pozo W1 en la capa A tiene asignado el bloque AA110, el correlativo que se escribira en la tabla t2wellblock es A110.


	Examples
	--------
	>>>  insert_wellblock_to_sqlite(db_path,'../input/',wells)
	"""

	if os.path.isfile('../mesh/wells_correlative.txt'):

		with open('../mesh/wells_correlative.txt',encoding='utf8') as json_file:
		    wells_correlative=json.load(json_file)

		conn=sqlite3.connect(db_path)
		c=conn.cursor()

		types=['WELLS','MAKE_UP_WELLS','NOT_PRODUCING_WELL']
		wells=[]
		for scheme in types:
			try:
				for well in input_dictionary[scheme]:
					wells.append(well)
			except KeyError:
					pass

		for name in wells:
			try:
				blocknumer=wells_correlative[name][0]
				q="INSERT INTO t2wellblock(well,blockcorr) VALUES ('%s','%s')"%(name,blocknumer)
				c.execute(q)
				conn.commit()
			except KeyError:
				pass
			except sqlite3.IntegrityError:
				print("The block  number is already assing for the well: %s"%name)
		conn.close()

	else:
		print("The file well_correlative.txt do not exists")

def insert_layers_to_sqlite(input_dictionary=input_data):
	"""Escribe capas y sus elevaciones

	Parameters
	----------
	db_path : str
	  Direccion de base de datos sqlite, tomado de model_conf

	Note
	----
	Toma los datos del diccionario layers definido en model_conf

	Examples
	--------
	>>>  insert_layers_to_sqlite(db_path)
	"""

	layers_info=geomtr.vertical_layers(input_dictionary)
	top=layers_info['top']
	correlative=layers_info['name']
	middle=layers_info['middle']
	bottom=layers_info['bottom']

	conn=sqlite3.connect(input_dictionary['db_path'])
	c=conn.cursor()

	for n in range(len(top)):
		q="INSERT INTO layers(correlative,top,middle,bottom) VALUES ('%s',%s,%s,%s)"\
			%(correlative[n],top[n],middle[n],bottom[n])
		try:
			c.execute(q)
			conn.commit()
		except sqlite3.IntegrityError:
			print("The layer %s is already define, correlative is a PRIMARY KEY"%correlative[n])
	conn.close()


"""
Chinameca
"""


"""
db_creation(db_path)


insert_survey_to_sqlite(db_path,source_txt)
insert_wells_sqlite(db_path,source_txt)
insert_PT_to_sqlite(db_path,source_txt)
insert_drawdown_to_sqlite(db_path,source_txt)
insert_cooling_to_sqlite(db_path,source_txt)
insert_layers_to_sqlite(db_path)
"""

"""
insert_feedzone_to_sqlite(db_path,source_txt)


insert_wellblock_to_sqlite(db_path,source_txt,wells)
insert_layers_to_sqlite(db_path,source_txt)

insert_mh_to_sqlite(db_path,source_txt)
"""


#sqlite_to_json(db_path,source_txt)
#insert_wellblock_to_sqlite(db_path,source_txt,wells)
#insert_feedzone_to_sqlite(db_path,source_txt)
#insert_mh_to_sqlite(db_path,source_txt)


#Sostenibilidad Ahuachapan

"""
db_creation(db_path)
insert_survey_to_sqlite(db_path,source_txt)
insert_wells_sqlite(db_path,source_txt)
insert_PT_to_sqlite(db_path,source_txt)
insert_drawdown_to_sqlite(db_path,source_txt)
insert_cooling_to_sqlite(db_path,source_txt)
insert_mh_to_sqlite(db_path,source_txt)


insert_layers_to_sqlite(db_path) #Actualizar
insert_feedzone_to_sqlite(db_path,source_txt)
insert_wellblock_to_sqlite(db_path,source_txt,wells)

"""

#Cuando se crean los escenarios es necesario borrar los flujo previos para cuando el script lea la base para crear el GENER lea segun se han los flujos nuevos

#insert_mh_to_sqlite(db_path,source_txt)

"""
db_creation()

insert_survey_to_sqlite()
insert_wells_sqlite()
insert_PT_to_sqlite()
insert_drawdown_to_sqlite()
insert_cooling_to_sqlite()
insert_mh_to_sqlite()


insert_layers_to_sqlite() #Actualizar
insert_feedzone_to_sqlite()
insert_wellblock_to_sqlite()

"""

#insert_PT_to_sqlite()
#insert_layers_to_sqlite()


from distutils.core import setup

setup(
    name='T2GEORES',
    version='0.1.2',
    packages=['T2GEORES',],
    license='MIT',
    author='Erick Jiménez',
    author_email="erick@grogtp.is",
    long_description=open('README.txt').read(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "Development Status :: 3 - Alpha",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    install_requires=[
        'pandas==1.1.3',
        'numpy>=1.14.5',
        'matplotlib>=3.3.2',
        'iapws>=1.5.2',
		'scipy>=1.5.3'
    ],
)


from model_conf import input_data
from formats import formats_t2
import numpy as np
import csv
from scipy import interpolate
import pandas as pd
import sys
import shapefile
import json
from iapws import IAPWS97

def vertical_layers(input_dictionary=input_data):
	"""Regresa informacion relacionada a las capas

	Returns
	-------
	array
	  Elevacion superior de capa
	array
	  Nombre de capa
	array
	  Elevacion media de capa
	array
	  Elevacion inferior de capa
	
	Examples
	--------
	>>> vertical_layer(layers,z0_level)
	"""

	layers_info={'name':[],'top':[],'middle':[],'bottom':[]}

	depth=0

	layers=input_dictionary['LAYERS']
	for cnt, layer in enumerate(sorted(layers)):
		depth+=layers[layer][1]
		z_bottom=input_dictionary['z_ref']-depth
		if cnt==0:
			z_top=input_dictionary['z_ref']
		else:
			z_top+=-layers[last_key][1]
		last_key=layer

		z_mid=(z_top+z_bottom)*0.5

		layers_info['name'].append(layers[layer][0])
		layers_info['top'].append(z_top)
		layers_info['middle'].append(z_mid)
		layers_info['bottom'].append(z_bottom)

	return layers_info

def MD_to_TVD(well,depth):
	"""Retorna las posiciones xyz de una profundidad especifica

	Parameters
	----------
	well : str
	  Pozo seleccionado
	depth : float
	  Profundidad seleccionada para extraccion de informacion


	Returns
	-------
	float
	  x_V : posicion x de profundidad especificada
	float
	  y_V :  posicion y de profundidad especificada
	float
	  z_V :  posicion z de profundidad especificada
	  
	Attention
	---------
	Los archivos input/ubication.csv e input/survey/{pozo}_MD.dat deben existir

	Note
	----
	Se utiliza una regresion lineal entre cada punto del registro

	Examples
	--------
	>>> MD_to_TVD('AH-1',500)
	"""
	file="../input/survey/%s_MD.dat"%well
	MD,DeltaY,DeltaX=np.loadtxt(file,skiprows=1,unpack=True,delimiter=',')

	reader = csv.DictReader(open("../input/ubication.csv", 'r')) #'rb'
	dict_ubication={}
	for line in reader:
		dict_ubication[line['well']]=line
	z_0=float(dict_ubication[well]['masl'])
	x_0=float(dict_ubication[well]['east'])
	y_0=float(dict_ubication[well]['north'])

	#Initialize the delta z values
	z_delta=[0 for i in MD]
	x=[0 for i in MD]
	y=[0 for i in MD]
	z=[0 for i in MD]

	#Assuming straight line between points
	for j in range(len(MD)):
		if j==0:
			z_delta[j]=0
		else:
			z_delta[j]=((MD[j]-MD[j-1])**2-(DeltaX[j]-DeltaX[j-1])**2-(DeltaY[j]-DeltaY[j-1])**2)**0.5+z_delta[j-1]
	
	#Convertion delta to absolute
	for j in range(len(MD)):
		x[j]=x_0+DeltaX[j]
		y[j]=y_0+DeltaY[j]
		z[j]=z_0-z_delta[j]

	#Function of X-Y-Z with MD
	funxmd=interpolate.interp1d(MD,x)
	funymd=interpolate.interp1d(MD,y)
	funzmd=interpolate.interp1d(MD,z)

	try:
		x_out=funxmd(depth)
		y_out=funymd(depth)
		z_out=funzmd(depth)
	except ValueError:
		x_out=np.nan
		y_out=np.nan
		z_out=np.nan
	return x_out,y_out,z_out

def MD_to_TVD_one_var_array(well,var_array,MD_array,num_points):
	"""Encuentra la posicion xyz de cada punto a lo largo de un registro,  junto con la temperatura asociada

	Parameters
	----------
	well : str
	  Pozo seleccionado
	var_array : array
	  Arreglo que contiene registro de valores de Presion o temperatura
	MD_array   :array
	  Arreglo que contiene registro de valores de profundidad medida
	num_points	:int
	  Numero de puntos a interpolar

	Returns
	-------
	array
	  x_V : arreglo con posicion x de cada punto del registro
	array
	  y_V : arreglo con posicion y de cada punto del registro
	array
	  z_V : arreglo con posicion z de cada punto del registro
	array
	  v_V : arreglo con valores de Presion o Temperatura para cada punto xyz
	  
	Attention
	---------
	Los archivos input/ubication.csv e input/survey/{pozo}_MD.dat deben existir

	Note
	----
	Se utiliza una regresion lineal entre cada punto del registro

	Examples
	--------
	>>> MD_to_TVD_one_var_array('AH-16',data['Temp'],data['MD'],100)
	"""

	file="../input/survey/%s_MD.dat"%well
	MD,DeltaY,DeltaX=np.loadtxt(file,skiprows=1,unpack=True,delimiter=',')
	print(well)

	reader = csv.DictReader(open("../input/ubication.csv", 'r'))
	dict_ubication={}
	for line in reader:
		dict_ubication[line['well']]=line
	z_0=float(dict_ubication[well]['masl'])
	x_0=float(dict_ubication[well]['east'])
	y_0=float(dict_ubication[well]['north'])

	#Initialize the delta z values
	z_delta=[0 for i in MD]
	x=[0 for i in MD]
	y=[0 for i in MD]
	z=[0 for i in MD]

	#Assuming straight line between points
	for j in range(len(MD)):
		if j==0:
			z_delta[j]=0
		else:
			z_delta[j]=((MD[j]-MD[j-1])**2-(DeltaX[j]-DeltaX[j-1])**2-(DeltaY[j]-DeltaY[j-1])**2)**0.5+z_delta[j-1]
	
	#Convertion delta to absolute
	for j in range(len(MD)):
		x[j]=x_0+DeltaX[j]
		y[j]=y_0+DeltaY[j]
		z[j]=z_0-z_delta[j]

	#Function of X-Y-Z with MD
	funxmd=interpolate.interp1d(MD,x)
	funymd=interpolate.interp1d(MD,y)
	funzmd=interpolate.interp1d(MD,z)

	#Working with variable

	funcV=interpolate.interp1d(MD_array,var_array)
	x_V=[]
	y_V=[]
	z_V=[]
	var_V=[]

	MD_array_reconvert=np.linspace(min(MD_array),max(MD_array),num_points)

	for i in MD_array_reconvert:
		try:
			x_V.append(funxmd(i))
			y_V.append(funymd(i))
			z_V.append(funzmd(i))
			var_V.append(funcV(i))
		except ValueError:
			pass

	return x_V,y_V,z_V,var_V

def TVD_to_MD(well,TVD):
	"""Encuentra la profundidad medida para un vapor de msnm solicitado

	Parameters
	----------
	well : str
	  Pozo seleccionado
	TVD : float
	  msnm de profundidad medida solicitada

	Returns
	-------
	float
	  MD : profundidad medida
	  
	Attention
	---------
	Los archivos input/ubication.csv e input/survey/{pozo}_MD.dat deben existir

	Note
	----
	Se utiliza una regresion lineal entre cada punto del registro

	Examples
	--------
	>>> TVD_to_MD('AH-16',-100)
	"""

	file="../input/survey/%s_MD.dat"%well
	MD,DeltaY,DeltaX=np.loadtxt(file,skiprows=1,unpack=True,delimiter=',')

	reader = csv.DictReader(open("../input/ubication.csv", 'r')) #'rb'
	dict_ubication={}
	for line in reader:

		dict_ubication[line['well']]=line
	z_0=float(dict_ubication[well]['masl'])
	x_0=float(dict_ubication[well]['east'])
	y_0=float(dict_ubication[well]['north'])

	#Initialize the delta z values
	z_delta=[0 for i in MD]
	x=[0 for i in MD]
	y=[0 for i in MD]
	z=[0 for i in MD]

	#Assuming straight line between points
	for j in range(len(MD)):
		if j==0:
			z_delta[j]=0
		else:
			z_delta[j]=((MD[j]-MD[j-1])**2-(DeltaX[j]-DeltaX[j-1])**2-(DeltaY[j]-DeltaY[j-1])**2)**0.5+z_delta[j-1]
	
	#Convertion delta to absolute
	for j in range(len(MD)):
		z[j]=z_0-z_delta[j]

	#Function of X-Y-Z with MD
	funzmd=interpolate.interp1d(z,MD)


	try:
		MD=funzmd(TVD)
	except ValueError:
		MD=np.nan
	return MD

def TVD_to_MD_array(well,TVD_array):
	"""Retorna las profundidades medidas para una serie de valores TVD

	Parameters
	----------
	well : str
	  Pozo seleccionado
	TVD_array : array
	  Arreglo de msnm

	Returns
	-------
	array
	  MD : arreglo de profundidades medidas
	  
	Attention
	---------
	Los archivos input/ubication.csv e input/survey/{pozo}_MD.dat deben existir

	Note
	----
	Se utiliza una regresion lineal entre cada punto del registro

	Examples
	--------
	>>> TVD_to_MD_array('AH-34',data['TVD'])
	"""

	file="../input/survey/%s_MD.dat"%well
	MD,DeltaY,DeltaX=np.loadtxt(file,skiprows=1,unpack=True,delimiter=',')

	reader = csv.DictReader(open("../input/ubication.csv", 'r')) #'rb'
	dict_ubication={}
	for line in reader:

		dict_ubication[line['well']]=line
	z_0=float(dict_ubication[well]['masl'])
	x_0=float(dict_ubication[well]['east'])
	y_0=float(dict_ubication[well]['north'])

	#Initialize the delta z values
	z_delta=[0 for i in MD]
	x=[0 for i in MD]
	y=[0 for i in MD]
	z=[0 for i in MD]

	#Assuming straight line between points
	for j in range(len(MD)):
		if j==0:
			z_delta[j]=0
		else:
			z_delta[j]=((MD[j]-MD[j-1])**2-(DeltaX[j]-DeltaX[j-1])**2-(DeltaY[j]-DeltaY[j-1])**2)**0.5+z_delta[j-1]
	
	#Convertion delta to absolute
	for j in range(len(MD)):
		z[j]=z_0-z_delta[j]

	#Function of X-Y-Z with MD
	funzmd=interpolate.interp1d(z,MD)

	MD=[]
	for v in TVD_array:
		try:
			MD.append(funzmd(v))
		except ValueError:
			MD.append(np.nan)
	return MD

def line_intersect(Ax1, Ay1, Ax2, Ay2, Bx1, By1, Bx2, By2):
	"""Encentra el punto de interseccion de dos lineas

	Parameters
	----------	  
	{letter}{position}{corr} : float
	  Identifica el punto A o B, la posicion ( x o y ) y el correlativo (1 o 2)

	Returns
	-------
	tuple
	  par (x,y), de no encontrar interseccion, returna none
	"""

	d = (By2 - By1) * (Ax2 - Ax1) - (Bx2 - Bx1) * (Ay2 - Ay1)
	if d:
		uA = ((Bx2 - Bx1) * (Ay1 - By1) - (By2 - By1) * (Ax1 - Bx1)) / d
		uB = ((Ax2 - Ax1) * (Ay1 - By1) - (Ay2 - Ay1) * (Ax1 - Bx1)) / d
	else:
		return
	if not(0 <= uA <= 1 and 0 <= uB <= 1):
		return
	x = Ax1 + uA * (Ax2 - Ax1)
	y = Ay1 + uA * (Ay2 - Ay1)

	return x, y

#print(MD_to_TVD('AH-1',500))

def write_well_track(input_dictionary=input_data,source_txt='../input/'):
	"""Convierte el survey MD en TVD de todos los pozos

	Parameters
	----------	  
	source_txt : str
	  Ubicacion de archivos fuentes
	wells : array
	  Arreglo de nombre de pozos

	Returns
	-------
	file
	  {pozo}_xyz.csv: en direccion ../input/survey/xyz

	Examples
	--------
	>>> write_well_track(wells)
	"""
	types=['WELLS','MAKE_UP_WELLS','NOT_PRODUCING_WELL']
	wells=[]
	for scheme in types:
		try:
			for well in input_dictionary[scheme]:
				wells.append(well)
		except KeyError:
				pass

	for well in wells:
		try:
			well_survey=pd.read_csv(source_txt+'survey/'+well+"_MD.dat")
			file=open(source_txt+'survey/'+well+"_xyz.csv","w")
			string="x,y,z\n"
			file.write(string)
			for index,row in well_survey.iterrows():
				x,y,z=MD_to_TVD(well,row['MeasuredDepth'])
				string="%s,%s,%s\n"%(x,y,z)
				file.write(string)
			file.close()
		except FileNotFoundError:
			sys.exit("There is no MD file for the well %s"%well)

#write_well_track(['AH-34'])

def write_feedzone_position(source_txt='../input/'):
	"""Genera un archivo con formato json que contiene la presion y temperatura para todos los bloques de un pozo asociados a una posicion en el espacio

	Parameters
	----------	  
	source_txt : str
	  Ubicacion de archivos fuentes

	Returns
	-------
	file
	  well_feezone_xyz: en direccion ../input

	Note
	----
	Este archivo es utilizado por pyamesh para ubicar los pozos

	Examples
	--------
	>>> write_feedzone_position(source_txt='../input/')
	"""

	source_file=source_txt+'well_feedzone.csv'
	try:
		feedzones=pd.read_csv(source_file,delimiter=",")	
	except FileNotFoundError:
		sys.exit("There is no MD file for the well %s"%source_file)

	file=open(source_txt+'well_feedzone_xyz.csv','w')

	string="well,MD,x,y,z,type\n"
	file.write(string)
	for index,row in feedzones.iterrows():
		x,y,z=MD_to_TVD(row['well'],row['MeasuredDepth'])
		string="%s,%s,%s,%s,%s,%s\n"%(row['well'],row['MeasuredDepth'],x,y,z,row['type'])
		file.write(string)
	file.close()

#write_feedzone_position()

def PT_natural_to_GIS():
	"""Genera un archivo shapefile con informacion de presion y temperatura para cada cada pozo segun el registro seleccionado

	Returns
	-------
	file
	  layer_{corr}_masl_{masl}.shp: en direccion ../input/PT/GIS/

	Examples
	--------
	>>> PT_natural_to_GIS()
	"""
	layers_info=vertical_layers()

	layer_dict={}
	for v in range(len(layers_info['middle'])):
		layer_dict[layers_info['middle'][v]]=layers_info['name'][v]

	for elevation in layers_info['middle']:
		w=shapefile.Writer('../input/PT/GIS/layer_%s_masl_%s.shp'%(layer_dict[elevation],elevation))
		w.field('TEMPERATURE','F',decimal=2)
		w.field('PRESSURE','F',decimal=2)
		for well in sorted(input_data['WELLS']):
			file_source_name="../input/PT/%s_MDPT.dat"%well
			try:
				data=pd.read_csv(file_source_name)
				fun_T=interpolate.interp1d(data['MD'],data['T'])
				fun_P=interpolate.interp1d(data['MD'],data['P'])
				try:
					masl_MD=TVD_to_MD(well,elevation)
					x,y,z=MD_to_TVD(well,masl_MD)
					if float(fun_T(masl_MD)) and float(fun_P(masl_MD)):
						w.point(x,y)
						w.record(float(fun_T(masl_MD)),float(fun_P(masl_MD)))
				except (ValueError,shapefile.ShapefileException) :
					pass

			except (IOError,shapefile.ShapefileException):
				pass
		w.close()

def PT_real_to_json():
	"""Genera un archivo con formato json que contiene la presion y temperatura para todos los bloques de un pozo asociados a una posicion en el espacio

	Returns
	-------
	file
	  PT_real_json: en direccion ../input/PT/PT_real_json.txt

	Examples
	--------
	>>> PT_real_to_json()
	"""

	layers_info=vertical_layers()

	layer_dict={}
	for v in range(len(layers_info['middle'])):
		layer_dict[layers_info['middle'][v]]=layers_info['name'][v]

	eleme_dict={}

	for well in sorted(input_data['WELLS']):
		file_source_name="../input/PT/%s_MDPT.dat"%well
		try:
			data=pd.read_csv(file_source_name)
			fun_T=interpolate.interp1d(data['MD'],data['T'])
			fun_P=interpolate.interp1d(data['MD'],data['P'])
			for masl in layers_info['middle']:
				masl_MD=TVD_to_MD(well,masl)
				try:
					if not np.isnan(masl_MD):
						x,y,z=MD_to_TVD(well,masl_MD)
						eleme_dict["%s-%s"%(well,layer_dict[masl])]=[float(x),float(y),masl,float(fun_T(masl_MD)),float(fun_P(masl_MD))]
				except ValueError:
					pass
		except IOError:
			pass

	with open("../input/PT/PT_real_json.txt",'w') as json_file:
		  json.dump(eleme_dict, json_file,sort_keys=True, indent=1)

def P_from_T_TVD(T_array,TVD_array,water_level_TVD,Pmin):
	"""Genera un perfil de presion basado en un registro de temperatura y un nivel hidroestatico

	Parameters
	----------
	T_array : array
	  Arreglo de temperatura
	TVD_array : array
	  Arreglo de profundidades verticales
	water_level_TVD : float
	  Profundiadad de nivel hidroestatico

	Returns
	-------
	float
	  P: arreglo de presion 

	Examples
	--------
	>>> P_from_Tlogging_TVD(T_array,TVD_array,water_level_TVD,Pmin)
	"""

	P=[]
	g=formats_t2['PARAMETERS']['GF']
	for n in range(len(TVD_array)):
		if TVD_array[n]<=water_level_TVD:
			if n==0:
				P.append(Pmin)
			else:
				try:
					C0=IAPWS97(T=(T_array[n]+273.15),x=0)
					P.append((water_level_TVD-TVD_array[n])*g*C0.rho/1E5+Pmin)
				except NotImplementedError:
					P.append(P[n-1])
		else:
			P.append(Pmin)
	return P





import os
from model_conf import *
import subprocess
import shutil
import t2_writer as t2w
import t2gener as t2g
from model_conf import input_data,geners

def incon_replace(incon_file,blocks,incon_file_len):
	string=""
	for index, incon_line in enumerate(incon_file):
		if blocks=='even' and index!=0 and index<=incon_file_len-3:
			if index%2==1:
				string+=incon_line[0:5]+'\n'
			else:
				string+=incon_line
		if blocks=='odd' :
			if index%2!=1:
				string+=incon_line
			else:
				string+=incon_line
	return string

def incon_delete():
	final_t2=""
	incon_section=False
	input_fi_file="../model/t2/t2"
	if os.path.isfile(input_fi_file):
		t2_file=open(input_fi_file, "r")
		for t2_line in t2_file:
			if 'INCON' in t2_line:
				incon_section=True
			elif 'ELEME' in t2_line:
				incon_section=False
			if not incon_section:
				final_t2+=t2_line
		t2_file.close()
		t2_file_out=open(input_fi_file, "w")
		t2_file_out.write(final_t2)
		t2_file_out.close()	
	else:
		sys.exit("The file %s or directory do not exist"%input_fi_file)

def incon_to_t2(incon_state='current'):
	"""Introduce el bloque incon al archivo TOUGH2, proviniente del archivo .sav

	Returns
	-------
	file
	  t2: modifica el archivo '../model/t2/t2' e introduce o actualiza el bloque INCON.

	Examples
	--------
	>>> incon_to_t2()
	"""

	#incon_state could be init or current

	if incon_state=='current':
		input_incon="../model/t2/t2.sav1"
		blocks='even'
		incon_file_len=len(open(input_incon).readlines())
	elif incon_state=='init':
		input_incon="../model/t2/sources/INCON"
		blocks='odd'
		incon_file_len=len(open(input_incon).readlines())

	input_fi_file="../model/t2/t2"

	if os.path.isfile(input_fi_file):
		if os.path.isfile(input_incon):
			incon_file=open(input_incon, "r")
			t2_file=open(input_fi_file, "r")
			if 'INCON' not in open(input_fi_file).read():
				final_t2=""
				for t2_line in t2_file:
					if 'ELEME' not in t2_line:
						final_t2+=t2_line
					elif 'ELEME' in t2_line:
						final_t2+='INCON\n'
						final_t2+=incon_replace(incon_file,blocks,incon_file_len)
						final_t2+='\nELEME----1----*----2----*----3----*----4----*----5----*----6----*----7----*----8\n'
			else:
				final_t2=""
				incon_section=False
				for t2_line in t2_file:
					if 'INCON' in t2_line:
						incon_section=True
					elif 'ELEME' in t2_line:
						incon_section=False
						final_t2+='INCON\n'
						final_t2+=incon_replace(incon_file,blocks,incon_file_len)
						final_t2+='\n'
					if not incon_section:
						final_t2+=t2_line

			incon_file.close()
			t2_file.close()
			t2_file_out=open(input_fi_file, "w")
			t2_file_out.write(final_t2)
			t2_file_out.close()
		else:
			sys.exit("The file %s or directory do not exist"%input_fi_file)
	else:
		sys.exit("The file %s or directory do not exist"%input_fi_file)

def update_gen(type_run=input_data['TYPE_RUN']):
	"""Actualiza el archivo '../model/t2/sources/GENER_SOURCES', escribe los valores 
	actualizados en la base sqlite e introduce el valor bloque de texto en el archivo t2

	Parameters
	----------
	db_path : str
	  Direccion de base de datos sqlite, tomado de model_conf
	geners : dictionary
	  Diccionario que especifica un elemento del bloque GENER con condiciones constantes, es decir, en caso de ser tipo MASS solo admite un valor

	Returns
	-------
	file
	  GENER_SOURCES: archivo de texto en direccion '../model/t2/sources/GENER_SOURCES' contiene la informacion de cada elemento definido como fuente o sumidero en model_conf

	Examples
	--------
	>>> update_gen(db_path,geners)
	"""

	#t2g.write_geners_to_txt_and_sqlite()

	final_t2=""
	incon_section=False
	input_fi_file="../model/t2/t2"
	sources_file='../model/t2/sources/GENER_SOURCES'
	well_sources_file='../model/t2/sources/GENER_PROD'
	makeup_well_sources_file='../model/t2/sources/GENER_MAKEUP'

	if os.path.isfile(input_fi_file):
		if os.path.isfile(sources_file):
			if 'GENER' not in open(input_fi_file).read():
				t2_file=open(input_fi_file, "a")
				t2_file.write('\nGENER----1----*----2----*----3----*----4----*----5----*----6----*----7----*----8\n')
				
				gener_source_file=open(sources_file, "r")
				for sources in gener_source_file:
					t2_file.write(sources)

				if type_run=='production':
					if os.path.isfile(well_sources_file):
						well_source_file=open(well_sources_file, "r")
						for sources in well_source_file:
							t2_file.write(sources)
						well_source_file.close()
					else:
						print("The file %s or directory do not exist"%well_sources_file)

					if os.path.isfile(makeup_well_sources_file):
						well_makeup_source_file=open(makeup_well_sources_file, "r")
						for sources in well_makeup_source_file:
							t2_file.write(sources)
						well_makeup_source_file.close()
					else:
						print("The file %s or directory do not exist"%well_sources_file)

				t2_file.write('ENDCY----1----*----2----*----3----*----4----*----5----*----6----*----7----*----8\n')
				t2_file.close()
				gener_source_file.close()

			else:
				t2_string=""
				t2_file=open(input_fi_file, "r")
				for t2_line in t2_file:
					if 'GENER' not in t2_line:
						t2_string+=t2_line
					else:
						t2_string+='GENER----1----*----2----*----3----*----4----*----5----*----6----*----7----*----8\n'
						gener_source_file=open(sources_file, "r")
						for sources in gener_source_file:
							t2_string+=sources

						if type_run=='production':
							if os.path.isfile(well_sources_file):
								well_source_file=open(well_sources_file, "r")
								for sources in well_source_file:
									t2_string+=sources
								well_source_file.close()
							else:
								print("The file %s or directory do not exist"%well_sources_file)
						t2_string+='\n'
						if os.path.isfile(makeup_well_sources_file):
							well_makeup_source_file=open(makeup_well_sources_file, "r")
							for sources in well_makeup_source_file:
								t2_string+=sources
							well_makeup_source_file.close()
						else:
							print("The file %s or directory do not exist"%well_sources_file)

						break
					
				t2_string+='\nENDCY----1----*----2----*----3----*----4----*----5----*----6----*----7----*----8\n'
				t2_file.close()

				t2_file_out=open(input_fi_file, "w")
				t2_file_out.write(t2_string)
				t2_file_out.close()	
		else:
			sys.exit("The file %s or directory do not exist"%sources_file)
	else:
		sys.exit("The file %s or directory do not exist"%input_fi_file)
	#subprocess.Popen(["sh","shell\\insert_geners.sh"])
	#subprocess.call(['./shell/insert_geners.sh'])

def update_rock_distribution(include_FOFT,include_SOLVR,include_COFT,include_GOFT,include_RPCAP,include_MULTI,include_START,type_run=input_data['TYPE_RUN']):
	"""Actualiza el archivo '../model/t2/sources/GENER_SOURCES', escribe los valores 
	actualizados en la base sqlite e introduce el valor bloque de texto en el archivo t2

	Parameters
	----------
	db_path : str
	  Direccion de base de datos sqlite, tomado de model_conf
	geners : dictionary
	  Diccionario que especifica un elemento del bloque GENER con condiciones constantes, es decir, en caso de ser tipo MASS solo admite un valor
	param : dictionary
	  Diccionario que contiene los elementos del bloque PARAM definidos en model_conf
	multi : dictionary
	  Diccionario que contiene los elementos del bloque MULTI definidos en model_conf
	solver : dictionary
	  Diccionario que contiene los elementos del bloque SOLVER definidos en model_conf
	recap : dictionary
	  Diccionario que contiene los elementos del bloque RECAP definidos en model_conf
	type_run : str
	  Puede ser natural o production, el primero no introduce el archivo '../model/t2/sources/GENER_PROD' al archivo t2
	recap : str
	  Titulo del modelo, tomado de model_conf

	Returns
	-------
	file
	  t2: archivo de entrada TOUGH2

	Examples
	--------
	>>> update_rock_distribution(db_path,geners,param,multi,solver,recap,'natural',title)
	"""

	"""
	value = input("How many times have you run copy_folder?\n")
	if value==1:
	"""
	t2g.write_geners_to_txt_and_sqlite()
	t2w.CONNE_from_steinar_to_t2()
	t2w.merge_ELEME_and_in_to_t2()
	t2w.t2_input(include_FOFT,include_SOLVR,include_COFT,include_GOFT,include_RPCAP,include_MULTI,include_START)

def copy_folder(src,dest):
	"""Funcion generada para copiar todo el contenido de una carpeta en otra

	Parameters
	----------
	src : str
	  Direccion de folder fuente
	dest : str
	  Direccion de folder destino


	Examples
	--------
	>>> copy_folder('../model/t2','../model/t2/prev')
	"""
	src_files = os.listdir(src)
	for file_name in src_files:
		full_file_name = os.path.join(src, file_name)
		if os.path.isfile(full_file_name):
			try:
				shutil.copy(full_file_name, dest)
			except shutil.Error:
				print("%s and %s didnt change"%(full_file_name,dest))

def create_prev():
	"""Genera una copia de carpetas con resultados de corrida para posterior comparacion

	Note
	----
	Las carpetas a copiar son: 
	'../model/t2'
	'../output/PT/images'
	'../output/PT/json'
	'../output/PT/txt'
	'../output/PT/evol'
	'../output/mh'
	Todas son copiadas a un subfolder llamado prev

	Examples
	--------
	>>> create_prev()
	"""

	#Copy t2 output
	src='../model/t2'
	dest= '../model/t2/prev'
	copy_folder(src,dest)

	#Copy prev images output
	src='../output/PT/images'
	dest='../output/PT/images/prev'
	copy_folder(src,dest)

	#Copy prev json output
	src='../output/PT/json'
	dest='../output/PT/json/prev'
	copy_folder(src,dest)

	#Copy prev txt output
	src='../output/PT/txt'
	dest='../output/PT/txt/prev'
	copy_folder(src,dest)

	#Copy prev txt output
	src='../output/PT/evol'
	dest='../output/PT/evol/prev'
	copy_folder(src,dest)

	#Copy prev txt output
	src='../output/mh/txt'
	dest='../output/mh/txt/prev'
	copy_folder(src,dest)

def run(EOS=1):
	current=os.path.abspath(os.getcwd())
	t2_input_file='%s/model/t2/t2'%current.replace('/scripts','')
	it2_input_file='%s/model/t2/fit2'%current.replace('/scripts','')
	shutil.copyfile(t2_input_file, current+'/t2')
	shutil.copyfile(it2_input_file, current+'/fit2')

	subprocess.run(["itough2", 'fit2','t2',str(EOS)])

	target_dir = current.replace('/scripts','/model/t2')
	file_names = os.listdir(current)

	for file_name in file_names:
		if 't2.' in file_name or 'fit2' in file_name:
			try:
				shutil.move(os.path.join(current, file_name),os.path.join(target_dir, file_name) )
			except shutil.Error:
				print("File already exist %s"%file_name)

#Sostenibilidad Ahuachapan

#incon_from_st_to_t2()

#update_gen(db_path,geners)

#incon_to_t2()

#update_gen(db_path,geners)

#create_prev()

#incon_to_t2()
#incon_delete()
#update_gen()
#update_rock_distribution(include_FOFT=True,include_SOLVR=True,include_COFT=True,include_GOFT=True,include_RPCAP=True,include_MULTI=True,include_START=True)

#run()